#define noid_root 4007
#define noid_file 5113
#define noid_free 5433

typedef struct NOiD_fs
{
	unsigned long int id;		//permanent id , represent the real place in the array
	unsigned long int nfo;		//check sum id , defined higher
	unsigned long int file;		//file id , 	or file count if root
	unsigned long int misc;		//other info , 	or cluster size if root
	unsigned long int last;
	unsigned long int next;
	unsigned long int back;
	unsigned long int fore;
} NOiD_fs;
//===============================================================
//===============================================================
//                    4 NOOB
//===============================================================
//===============================================================
int NOiD_qty (int noid_size, int noid_cluster)
{	return (noid_size / noid_cluster) - 1;	}

int NOiD_size (int noid_qty)
{	return noid_qty * sizeof(NOiD_fs);	}

void * NOiD_cluster (int noid_numb, int noid_size,void * noid_src)
{	return noid_src + (noid_numb * noid_size);	}

void * NOiD_read (int noid_numb, int noid_size, void * noid_table, void * noid_dest)
{
	while (noid_size != 0)
	{
		*(char*)(noid_dest + noid_size-1) = *(char*)(((void*)noid_table) + (noid_numb * noid_size-1));
		noid_size -= 1;
	}
	return noid_dest;
}

void * NOiD_write (int noid_numb, int noid_size, void * noid_table, void * noid_src)
{
	while (noid_size != 0)
	{
		*(char*)(((void*)noid_table) + (noid_numb * noid_size-1)) = *(char*)(noid_src + noid_size-1);
		noid_size -= 1;
	}
	return noid_src;
}
//===============================================================
//===============================================================
//                     FORMAT
//===============================================================
//===============================================================
NOiD_fs * NOiD_build (int noid_qty, int noid_cluster, void * noid_ptr)
{
	if ((noid_qty == 0)||(noid_ptr == 0)||(noid_cluster <= sizeof(NOiD_fs)))
	{	return 0;	}

	int noid_tsize = noid_qty * sizeof(NOiD_fs);
	int noid_tend = (noid_tsize + (sizeof(NOiD_fs)-1)) / noid_cluster;
	int noid_start = noid_tend + 1;
	int noid_end = noid_qty;
	int noid_go = 0;

	while (noid_go <= noid_qty)
	{
		NOiD_fs * noid_new = (NOiD_fs*)(noid_ptr + (sizeof(NOiD_fs) * noid_go));
		noid_new->id = noid_go;
		if (noid_go >= noid_start)
		{
			noid_new->nfo = noid_free;
			noid_new->file = 1;
			noid_new->misc = 0;
			if (noid_go == noid_start)
			{	noid_new->last = noid_end;	}
			else
			{	noid_new->last = noid_go - 1;	}
			if (noid_go == noid_end)
			{	noid_new->next = noid_start;	}
			else
			{	noid_new->next = noid_go + 1;	}
			noid_new->back = 0;
			noid_new->fore = 0;
		}
		else
		{
			noid_new->nfo = noid_root;
			noid_new->file = 1;
			noid_new->misc = noid_cluster;
			if (noid_go == 0)
			{	noid_new->last = noid_tend;	}
			else
			{	noid_new->last = noid_go - 1;	}
			if (noid_go == noid_tend)
			{	noid_new->next = 0;		}
			else
			{	noid_new->next = noid_go + 1;	}
			noid_new->back = noid_start;
			noid_new->fore = noid_start;
		}
		noid_go += 1;
	}
	return (NOiD_fs*) noid_ptr;
}
//===============================================================
//===============================================================
//                     CLUSTER
//===============================================================
//===============================================================
int NOiD_mkfile (NOiD_fs * noid_table)
/* will make a new file completly from free cluster
   noid_table = pointer to the table in memory */
{
	if (noid_table->id != 0)
	{ 	return 999999;	}

	int noid_new_id, noid_free_id, noid_fore_id, noid_next_id;
	NOiD_fs * noid_new, * noid_free_file, * noid_fore_file, * noid_next_free;

	noid_free_id = noid_table->fore;	//first file of the table
	noid_free_file = (NOiD_fs*)(((void*)noid_table) + (noid_free_id * sizeof(NOiD_fs)));
	noid_fore_id = noid_free_file->fore;	//second file of the table
	noid_fore_file = (NOiD_fs*)(((void*)noid_table) + (noid_fore_id * sizeof(NOiD_fs)));
	if (noid_free_file->next != noid_free_file->id)
	{
		noid_new_id = noid_free_file->next;
	} else {
		return 999999;
	}
	noid_new = (NOiD_fs*)(((void*)noid_table) + (noid_new_id * sizeof(NOiD_fs)));
	noid_next_id = noid_new->next;
	noid_next_free = (NOiD_fs*)(((void*)noid_table) + (noid_next_id * sizeof(NOiD_fs)));

	noid_table->file += 1;
	noid_free_file->next = noid_new->next;
	noid_free_file->fore = noid_new->id;
	noid_next_free->last = noid_new->last;
	noid_fore_file->back = noid_new->id;

	noid_new->nfo = noid_file;
	noid_new->file = noid_table->file;
	noid_new->misc = 0;
	noid_new->last = noid_new->id;
	noid_new->next = noid_new->id;
	noid_new->back = noid_free_file->id;
	noid_new->fore = noid_fore_file->id;

	return noid_new_id;
}

void NOiD_rmfile (NOiD_fs * noid_table, int noid_numb)
/* will bring back file and all undo redo to free cluster list
   noid_table = pointer to the table in memory
   noid_numb = cluster permanent id */
{
	if ((noid_table->id != 0)||(noid_numb == 0)||(noid_numb == 1))
	{ 	return;	}
}
//===============================================================
//===============================================================
//                ORTHOGONAL PERSISTENCE
//                  @ CLUSTER LEVEL
//===============================================================
//===============================================================
int NOiD_pushfile (NOiD_fs * noid_table, int noid_numb)
/* will push ure file under undo list and then add new free cluster to file
   noid_table = pointer to the table in memory
   noid_numb = cluster permanent id of the file */
{
	if ((noid_table->id != 0)||(noid_numb == 0)||(noid_numb == 1))
	{ 	return 999999;	}

	return 0;
}

int NOiD_popfile (NOiD_fs * noid_table, int noid_numb)
/* will pop file to free while keeping undo redo , then 1st redo will take place
   noid_table = pointer to the table in memory
   noid_numb = cluster permanent id of the file */
{
	if ((noid_table->id != 0)||(noid_numb == 0)||(noid_numb == 1))
	{ 	return 999999;	}

	return 0;
}


int NOiD_undofile (NOiD_fs * noid_table, int noid_numb)
/* will rotate file list to give next undo first access
   noid_table = pointer to the table in memory
   noid_numb = cluster permanent id of the file */
{
	if ((noid_table->id != 0)||(noid_numb == 0)||(noid_numb == 1))
	{ 	return 999999;	}

	return 0;
}

int NOiD_redofile (NOiD_fs * noid_table, int noid_numb)
/* will rotate file list to give next redo first access
   noid_table = pointer to the table in memory
   noid_numb = cluster permanent id of the file */
{
	if ((noid_table->id != 0)||(noid_numb == 0)||(noid_numb == 1))
	{ 	return 999999;	}

	return 0;
}

//=======================================================================
